@@ -16,6 +16,10 @@ from openstack import proxy
 
 
 class Proxy(proxy.Proxy):
+    _resource_registry = {
+        "resource_class": _resource_class.ResourceClass,
+        "resource_provider": _resource_provider.ResourceProvider,
+    }
 
     # resource classes
 
