@@ -22,42 +22,45 @@ class TestSecurityGroup(base.BaseFunctionalTest):
     def setUp(self):
         super(TestSecurityGroup, self).setUp()
         self.NAME = self.getUniqueString()
-        sot = self.conn.network.create_security_group(name=self.NAME)
+        sot = self.user_cloud.network.create_security_group(name=self.NAME)
         assert isinstance(sot, security_group.SecurityGroup)
         self.assertEqual(self.NAME, sot.name)
         self.ID = sot.id
 
     def tearDown(self):
-        sot = self.conn.network.delete_security_group(
-            self.ID, ignore_missing=False)
+        sot = self.user_cloud.network.delete_security_group(
+            self.ID, ignore_missing=False
+        )
         self.assertIsNone(sot)
         super(TestSecurityGroup, self).tearDown()
 
     def test_find(self):
-        sot = self.conn.network.find_security_group(self.NAME)
+        sot = self.user_cloud.network.find_security_group(self.NAME)
         self.assertEqual(self.ID, sot.id)
 
     def test_get(self):
-        sot = self.conn.network.get_security_group(self.ID)
+        sot = self.user_cloud.network.get_security_group(self.ID)
         self.assertEqual(self.NAME, sot.name)
         self.assertEqual(self.ID, sot.id)
 
     def test_list(self):
-        names = [o.name for o in self.conn.network.security_groups()]
+        names = [o.name for o in self.user_cloud.network.security_groups()]
         self.assertIn(self.NAME, names)
 
     def test_list_query_list_of_ids(self):
-        ids = [o.id for o in self.conn.network.security_groups(id=[self.ID])]
+        ids = [
+            o.id for o in self.user_cloud.network.security_groups(id=[self.ID])
+        ]
         self.assertIn(self.ID, ids)
 
     def test_set_tags(self):
-        sot = self.conn.network.get_security_group(self.ID)
+        sot = self.user_cloud.network.get_security_group(self.ID)
         self.assertEqual([], sot.tags)
 
-        self.conn.network.set_tags(sot, ['blue'])
-        sot = self.conn.network.get_security_group(self.ID)
-        self.assertEqual(['blue'], sot.tags)
+        self.user_cloud.network.set_tags(sot, ["blue"])
+        sot = self.user_cloud.network.get_security_group(self.ID)
+        self.assertEqual(["blue"], sot.tags)
 
-        self.conn.network.set_tags(sot, [])
-        sot = self.conn.network.get_security_group(self.ID)
+        self.user_cloud.network.set_tags(sot, [])
+        sot = self.user_cloud.network.get_security_group(self.ID)
         self.assertEqual([], sot.tags)
