@@ -23,6 +23,7 @@ from testscenarios import load_tests_apply_scenarios as load_tests  # noqa
 
 from openstack.cloud import exc
 from openstack import exceptions
+from openstack.network.v2 import port as _port
 from openstack.tests import fakes
 from openstack.tests.unit import base
 
@@ -1062,13 +1063,12 @@ class TestBaremetalNode(base.IronicTestCase):
 
     def test_register_machine(self):
         mac_address = '00:01:02:03:04:05'
-        nics = [{'mac': mac_address}]
+        nics = [{'address': mac_address}]
         node_uuid = self.fake_baremetal_node['uuid']
         # TODO(TheJulia): There is a lot of duplication
         # in testing creation. Surely this hsould be a helper
         # or something. We should fix this.
         node_to_post = {
-            'chassis_uuid': None,
             'driver': None,
             'driver_info': None,
             'name': self.fake_baremetal_node['name'],
@@ -1094,7 +1094,8 @@ class TestBaremetalNode(base.IronicTestCase):
         ])
         return_value = self.cloud.register_machine(nics, **node_to_post)
 
-        self.assertDictEqual(self.fake_baremetal_node, return_value)
+        self.assertEqual(self.uuid, return_value.id)
+        self.assertSubdict(self.fake_baremetal_node, return_value)
         self.assert_calls()
 
     # TODO(TheJulia): We need to de-duplicate these tests.
@@ -1103,7 +1104,7 @@ class TestBaremetalNode(base.IronicTestCase):
     # accounted for newer microversions.
     def test_register_machine_enroll(self):
         mac_address = '00:01:02:03:04:05'
-        nics = [{'mac': mac_address}]
+        nics = [{'address': mac_address, 'pxe_enabled': False}]
         node_uuid = self.fake_baremetal_node['uuid']
         node_to_post = {
             'chassis_uuid': None,
@@ -1124,13 +1125,6 @@ class TestBaremetalNode(base.IronicTestCase):
                     resource='nodes'),
                 validate=dict(json=node_to_post),
                 json=self.fake_baremetal_node),
-            dict(
-                method='POST',
-                uri=self.get_mock_url(
-                    resource='ports'),
-                validate=dict(json={'address': mac_address,
-                                    'node_uuid': node_uuid}),
-                json=self.fake_baremetal_port),
             dict(
                 method='PUT',
                 uri=self.get_mock_url(
@@ -1145,11 +1139,13 @@ class TestBaremetalNode(base.IronicTestCase):
                     append=[self.fake_baremetal_node['uuid']]),
                 json=manageable_node),
             dict(
-                method='GET',
+                method='POST',
                 uri=self.get_mock_url(
-                    resource='nodes',
-                    append=[self.fake_baremetal_node['uuid']]),
-                json=manageable_node),
+                    resource='ports'),
+                validate=dict(json={'address': mac_address,
+                                    'node_uuid': node_uuid,
+                                    'pxe_enabled': False}),
+                json=self.fake_baremetal_port),
             dict(
                 method='PUT',
                 uri=self.get_mock_url(
@@ -1163,19 +1159,7 @@ class TestBaremetalNode(base.IronicTestCase):
                     resource='nodes',
                     append=[self.fake_baremetal_node['uuid']]),
                 json=available_node),
-            dict(
-                method='GET',
-                uri=self.get_mock_url(
-                    resource='nodes',
-                    append=[self.fake_baremetal_node['uuid']]),
-                json=available_node),
         ])
-        # NOTE(When we migrate to a newer microversion, this test
-        # may require revision. It was written for microversion
-        # ?1.13?, which accidently got reverted to 1.6 at one
-        # point during code being refactored soon after the
-        # change landed. Presently, with the lock at 1.6,
-        # this code is never used in the current code path.
         return_value = self.cloud.register_machine(nics, **node_to_post)
 
         self.assertSubdict(available_node, return_value)
@@ -1183,7 +1167,7 @@ class TestBaremetalNode(base.IronicTestCase):
 
     def test_register_machine_enroll_wait(self):
         mac_address = self.fake_baremetal_port
-        nics = [{'mac': mac_address}]
+        nics = [{'address': mac_address}]
         node_uuid = self.fake_baremetal_node['uuid']
         node_to_post = {
             'chassis_uuid': None,
@@ -1204,19 +1188,6 @@ class TestBaremetalNode(base.IronicTestCase):
                     resource='nodes'),
                 validate=dict(json=node_to_post),
                 json=self.fake_baremetal_node),
-            dict(
-                method='POST',
-                uri=self.get_mock_url(
-                    resource='ports'),
-                validate=dict(json={'address': mac_address,
-                                    'node_uuid': node_uuid}),
-                json=self.fake_baremetal_port),
-            dict(
-                method='GET',
-                uri=self.get_mock_url(
-                    resource='nodes',
-                    append=[self.fake_baremetal_node['uuid']]),
-                json=self.fake_baremetal_node),
             dict(
                 method='PUT',
                 uri=self.get_mock_url(
@@ -1236,6 +1207,13 @@ class TestBaremetalNode(base.IronicTestCase):
                     resource='nodes',
                     append=[self.fake_baremetal_node['uuid']]),
                 json=manageable_node),
+            dict(
+                method='POST',
+                uri=self.get_mock_url(
+                    resource='ports'),
+                validate=dict(json={'address': mac_address,
+                                    'node_uuid': node_uuid}),
+                json=self.fake_baremetal_port),
             dict(
                 method='PUT',
                 uri=self.get_mock_url(
@@ -1248,7 +1226,7 @@ class TestBaremetalNode(base.IronicTestCase):
                 uri=self.get_mock_url(
                     resource='nodes',
                     append=[self.fake_baremetal_node['uuid']]),
-                json=available_node),
+                json=manageable_node),
             dict(
                 method='GET',
                 uri=self.get_mock_url(
@@ -1264,7 +1242,7 @@ class TestBaremetalNode(base.IronicTestCase):
 
     def test_register_machine_enroll_failure(self):
         mac_address = '00:01:02:03:04:05'
-        nics = [{'mac': mac_address}]
+        nics = [{'address': mac_address}]
         node_uuid = self.fake_baremetal_node['uuid']
         node_to_post = {
             'chassis_uuid': None,
@@ -1276,7 +1254,7 @@ class TestBaremetalNode(base.IronicTestCase):
         self.fake_baremetal_node['provision_state'] = 'enroll'
         failed_node = self.fake_baremetal_node.copy()
         failed_node['reservation'] = 'conductor0'
-        failed_node['provision_state'] = 'verifying'
+        failed_node['provision_state'] = 'enroll'
         failed_node['last_error'] = 'kaboom!'
         self.register_uris([
             dict(
@@ -1285,13 +1263,6 @@ class TestBaremetalNode(base.IronicTestCase):
                     resource='nodes'),
                 json=self.fake_baremetal_node,
                 validate=dict(json=node_to_post)),
-            dict(
-                method='POST',
-                uri=self.get_mock_url(
-                    resource='ports'),
-                validate=dict(json={'address': mac_address,
-                                    'node_uuid': node_uuid}),
-                json=self.fake_baremetal_port),
             dict(
                 method='PUT',
                 uri=self.get_mock_url(
@@ -1306,11 +1277,10 @@ class TestBaremetalNode(base.IronicTestCase):
                     append=[self.fake_baremetal_node['uuid']]),
                 json=failed_node),
             dict(
-                method='GET',
+                method='DELETE',
                 uri=self.get_mock_url(
                     resource='nodes',
-                    append=[self.fake_baremetal_node['uuid']]),
-                json=failed_node),
+                    append=[self.fake_baremetal_node['uuid']])),
         ])
 
         self.assertRaises(
@@ -1322,7 +1292,7 @@ class TestBaremetalNode(base.IronicTestCase):
 
     def test_register_machine_enroll_timeout(self):
         mac_address = '00:01:02:03:04:05'
-        nics = [{'mac': mac_address}]
+        nics = [{'address': mac_address}]
         node_uuid = self.fake_baremetal_node['uuid']
         node_to_post = {
             'chassis_uuid': None,
@@ -1342,13 +1312,6 @@ class TestBaremetalNode(base.IronicTestCase):
                     resource='nodes'),
                 json=self.fake_baremetal_node,
                 validate=dict(json=node_to_post)),
-            dict(
-                method='POST',
-                uri=self.get_mock_url(
-                    resource='ports'),
-                validate=dict(json={'address': mac_address,
-                                    'node_uuid': node_uuid}),
-                json=self.fake_baremetal_port),
             dict(
                 method='PUT',
                 uri=self.get_mock_url(
@@ -1361,13 +1324,12 @@ class TestBaremetalNode(base.IronicTestCase):
                 uri=self.get_mock_url(
                     resource='nodes',
                     append=[self.fake_baremetal_node['uuid']]),
-                json=self.fake_baremetal_node),
+                json=busy_node),
             dict(
-                method='GET',
+                method='DELETE',
                 uri=self.get_mock_url(
                     resource='nodes',
-                    append=[self.fake_baremetal_node['uuid']]),
-                json=busy_node),
+                    append=[self.fake_baremetal_node['uuid']])),
         ])
         # NOTE(TheJulia): This test shortcircuits the timeout loop
         # such that it executes only once. The very last returned
@@ -1384,7 +1346,7 @@ class TestBaremetalNode(base.IronicTestCase):
 
     def test_register_machine_enroll_timeout_wait(self):
         mac_address = '00:01:02:03:04:05'
-        nics = [{'mac': mac_address}]
+        nics = [{'address': mac_address}]
         node_uuid = self.fake_baremetal_node['uuid']
         node_to_post = {
             'chassis_uuid': None,
@@ -1394,6 +1356,8 @@ class TestBaremetalNode(base.IronicTestCase):
             'properties': None,
             'uuid': node_uuid}
         self.fake_baremetal_node['provision_state'] = 'enroll'
+        manageable_node = self.fake_baremetal_node.copy()
+        manageable_node['provision_state'] = 'manageable'
         self.register_uris([
             dict(
                 method='POST',
@@ -1401,19 +1365,6 @@ class TestBaremetalNode(base.IronicTestCase):
                     resource='nodes'),
                 json=self.fake_baremetal_node,
                 validate=dict(json=node_to_post)),
-            dict(
-                method='POST',
-                uri=self.get_mock_url(
-                    resource='ports'),
-                validate=dict(json={'address': mac_address,
-                                    'node_uuid': node_uuid}),
-                json=self.fake_baremetal_port),
-            dict(
-                method='GET',
-                uri=self.get_mock_url(
-                    resource='nodes',
-                    append=[self.fake_baremetal_node['uuid']]),
-                json=self.fake_baremetal_node),
             dict(
                 method='PUT',
                 uri=self.get_mock_url(
@@ -1421,12 +1372,37 @@ class TestBaremetalNode(base.IronicTestCase):
                     append=[self.fake_baremetal_node['uuid'],
                             'states', 'provision']),
                 validate=dict(json={'target': 'manage'})),
+            dict(
+                method='GET',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']]),
+                json=manageable_node),
+            dict(
+                method='POST',
+                uri=self.get_mock_url(
+                    resource='ports'),
+                validate=dict(json={'address': mac_address,
+                                    'node_uuid': node_uuid}),
+                json=self.fake_baremetal_port),
+            dict(
+                method='PUT',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid'],
+                            'states', 'provision']),
+                validate=dict(json={'target': 'provide'})),
             dict(
                 method='GET',
                 uri=self.get_mock_url(
                     resource='nodes',
                     append=[self.fake_baremetal_node['uuid']]),
                 json=self.fake_baremetal_node),
+            dict(
+                method='DELETE',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']])),
         ])
         self.assertRaises(
             exc.OpenStackCloudException,
@@ -1439,7 +1415,7 @@ class TestBaremetalNode(base.IronicTestCase):
 
     def test_register_machine_port_create_failed(self):
         mac_address = '00:01:02:03:04:05'
-        nics = [{'mac': mac_address}]
+        nics = [{'address': mac_address}]
         node_uuid = self.fake_baremetal_node['uuid']
         node_to_post = {
             'chassis_uuid': None,
@@ -1461,7 +1437,7 @@ class TestBaremetalNode(base.IronicTestCase):
                 uri=self.get_mock_url(
                     resource='ports'),
                 status_code=400,
-                json={'error': 'invalid'},
+                json={'error': 'no ports for you'},
                 validate=dict(json={'address': mac_address,
                                     'node_uuid': node_uuid})),
             dict(
@@ -1470,9 +1446,64 @@ class TestBaremetalNode(base.IronicTestCase):
                     resource='nodes',
                     append=[self.fake_baremetal_node['uuid']])),
         ])
-        self.assertRaises(exc.OpenStackCloudException,
-                          self.cloud.register_machine,
-                          nics, **node_to_post)
+        self.assertRaisesRegex(exc.OpenStackCloudException,
+                               'no ports for you',
+                               self.cloud.register_machine,
+                               nics, **node_to_post)
+
+        self.assert_calls()
+
+    def test_register_machine_several_ports_create_failed(self):
+        mac_address = '00:01:02:03:04:05'
+        mac_address2 = mac_address[::-1]
+        # Verify a couple of ways to provide MACs
+        nics = [mac_address, {'mac': mac_address2}]
+        node_uuid = self.fake_baremetal_node['uuid']
+        node_to_post = {
+            'chassis_uuid': None,
+            'driver': None,
+            'driver_info': None,
+            'name': self.fake_baremetal_node['name'],
+            'properties': None,
+            'uuid': node_uuid}
+        self.fake_baremetal_node['provision_state'] = 'available'
+        self.register_uris([
+            dict(
+                method='POST',
+                uri=self.get_mock_url(
+                    resource='nodes'),
+                json=self.fake_baremetal_node,
+                validate=dict(json=node_to_post)),
+            dict(
+                method='POST',
+                uri=self.get_mock_url(
+                    resource='ports'),
+                validate=dict(json={'address': mac_address,
+                                    'node_uuid': node_uuid}),
+                json=self.fake_baremetal_port),
+            dict(
+                method='POST',
+                uri=self.get_mock_url(
+                    resource='ports'),
+                status_code=400,
+                json={'error': 'no ports for you'},
+                validate=dict(json={'address': mac_address2,
+                                    'node_uuid': node_uuid})),
+            dict(
+                method='DELETE',
+                uri=self.get_mock_url(
+                    resource='ports',
+                    append=[self.fake_baremetal_port['uuid']])),
+            dict(
+                method='DELETE',
+                uri=self.get_mock_url(
+                    resource='nodes',
+                    append=[self.fake_baremetal_node['uuid']])),
+        ])
+        self.assertRaisesRegex(exc.OpenStackCloudException,
+                               'no ports for you',
+                               self.cloud.register_machine,
+                               nics, **node_to_post)
 
         self.assert_calls()
 
@@ -1656,9 +1687,10 @@ class TestBaremetalNode(base.IronicTestCase):
                 method='GET',
                 uri=self.get_mock_url(
                     service_type='network',
-                    resource='ports.json',
-                    base_url_append='v2.0'),
-                json={'ports': [{'id': vif_id}]}),
+                    resource='ports',
+                    base_url_append='v2.0',
+                    append=[vif_id]),
+                json={'id': vif_id}),
             dict(
                 method='POST',
                 uri=self.get_mock_url(
@@ -1682,9 +1714,10 @@ class TestBaremetalNode(base.IronicTestCase):
                 method='GET',
                 uri=self.get_mock_url(
                     service_type='network',
-                    resource='ports.json',
-                    base_url_append='v2.0'),
-                json={'ports': [{'id': vif_id}]}),
+                    resource='ports',
+                    base_url_append='v2.0',
+                    append=[vif_id]),
+                json={'id': vif_id}),
             dict(
                 method='DELETE',
                 uri=self.get_mock_url(
@@ -1716,14 +1749,17 @@ class TestBaremetalNode(base.IronicTestCase):
                 method='GET',
                 uri=self.get_mock_url(
                     service_type='network',
-                    resource='ports.json',
-                    base_url_append='v2.0'),
-                json={'ports': [fake_port]}),
+                    resource='ports',
+                    base_url_append='v2.0',
+                    append=[vif_id]),
+                json=fake_port),
         ])
         res = self.cloud.list_ports_attached_to_machine(
             self.fake_baremetal_node['uuid'])
         self.assert_calls()
-        self.assertEqual([fake_port], res)
+        self.assertEqual(
+            [_port.Port(**fake_port).to_dict(computed=False)],
+            [i.to_dict(computed=False) for i in res])
 
 
 class TestUpdateMachinePatch(base.IronicTestCase):
