@@ -16,18 +16,25 @@ test_create_server
 
 Tests for the `create_server` command.
 """
+
 import base64
 from unittest import mock
 import uuid
 
-from openstack import connection
 from openstack.cloud import exc
 from openstack.cloud import meta
+from openstack.compute.v2 import server
+from openstack import connection
 from openstack.tests import fakes
 from openstack.tests.unit import base
 
 
 class TestCreateServer(base.TestCase):
+    def _compare_servers(self, exp, real):
+        self.assertDictEqual(
+            server.Server(**exp).to_dict(computed=False),
+            real.to_dict(computed=False),
+        )
 
     def test_create_server_with_get_exception(self):
         """
@@ -40,6 +47,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -50,7 +58,8 @@ class TestCreateServer(base.TestCase):
                          u'imageRef': u'image-id',
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', '1234']),
@@ -73,6 +82,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -83,7 +93,8 @@ class TestCreateServer(base.TestCase):
                          u'imageRef': u'image-id',
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', '1234']),
@@ -106,6 +117,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -116,8 +128,8 @@ class TestCreateServer(base.TestCase):
                          u'imageRef': u'image-id',
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
-            self.get_nova_discovery_mock_dict(),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail']),
@@ -146,6 +158,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -156,8 +169,8 @@ class TestCreateServer(base.TestCase):
                          u'imageRef': u'image-id',
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
-            self.get_nova_discovery_mock_dict(),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail']),
@@ -183,6 +196,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -193,20 +207,20 @@ class TestCreateServer(base.TestCase):
                          u'imageRef': u'image-id',
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', '1234']),
                  json={'server': fake_server}),
         ])
-        normalized = self.cloud._expand_server(
-            self.cloud._normalize_server(fake_server), False, False)
-        self.assertEqual(
-            normalized,
+        self.assertDictEqual(
+            server.Server(**fake_server).to_dict(computed=False),
             self.cloud.create_server(
                 name='server-name',
                 image=dict(id='image-id'),
-                flavor=dict(id='flavor-id')))
+                flavor=dict(id='flavor-id')).to_dict(computed=False)
+        )
 
         self.assert_calls()
 
@@ -220,6 +234,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -231,21 +246,20 @@ class TestCreateServer(base.TestCase):
                          u'config_drive': True,
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', '1234']),
                  json={'server': fake_server}),
         ])
-        normalized = self.cloud._expand_server(
-            self.cloud._normalize_server(fake_server), False, False)
-        self.assertEqual(
-            normalized,
+        self.assertDictEqual(
+            server.Server(**fake_server).to_dict(computed=False),
             self.cloud.create_server(
                 name='server-name',
                 image=dict(id='image-id'),
                 flavor=dict(id='flavor-id'),
-                config_drive=True))
+                config_drive=True).to_dict(computed=False))
 
         self.assert_calls()
 
@@ -259,6 +273,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -269,21 +284,21 @@ class TestCreateServer(base.TestCase):
                          u'imageRef': u'image-id',
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', '1234']),
                  json={'server': fake_server}),
         ])
-        normalized = self.cloud._expand_server(
-            self.cloud._normalize_server(fake_server), False, False)
         self.assertEqual(
-            normalized,
+            server.Server(**fake_server).to_dict(computed=False),
             self.cloud.create_server(
                 name='server-name',
                 image=dict(id='image-id'),
                 flavor=dict(id='flavor-id'),
-                config_drive=None))
+                config_drive=None).to_dict(computed=False)
+        )
 
         self.assert_calls()
 
@@ -300,6 +315,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -311,18 +327,19 @@ class TestCreateServer(base.TestCase):
                          u'imageRef': u'image-id',
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', '1234']),
                  json={'server': fake_server}),
         ])
         self.assertEqual(
-            self.cloud._normalize_server(fake_create_server)['adminPass'],
+            admin_pass,
             self.cloud.create_server(
                 name='server-name', image=dict(id='image-id'),
                 flavor=dict(id='flavor-id'),
-                admin_pass=admin_pass)['adminPass'])
+                admin_pass=admin_pass)['admin_password'])
 
         self.assert_calls()
 
@@ -340,6 +357,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -351,13 +369,14 @@ class TestCreateServer(base.TestCase):
                          u'max_count': 1,
                          u'min_count': 1,
                          u'adminPass': admin_pass,
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
         ])
 
         # The wait returns non-password server
-        mock_wait.return_value = self.cloud._normalize_server(fake_server)
+        mock_wait.return_value = server.Server(**fake_server)
 
-        server = self.cloud.create_server(
+        new_server = self.cloud.create_server(
             name='server-name', image=dict(id='image-id'),
             flavor=dict(id='flavor-id'),
             admin_pass=admin_pass, wait=True)
@@ -367,8 +386,8 @@ class TestCreateServer(base.TestCase):
 
         # Even with the wait, we should still get back a passworded server
         self.assertEqual(
-            server['adminPass'],
-            self.cloud._normalize_server(fake_server_with_pass)['adminPass']
+            new_server['admin_password'],
+            fake_server_with_pass['adminPass']
         )
         self.assert_calls()
 
@@ -387,6 +406,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -398,7 +418,8 @@ class TestCreateServer(base.TestCase):
                          u'max_count': 1,
                          u'min_count': 1,
                          u'user_data': user_data_b64,
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', '1234']),
@@ -460,6 +481,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -470,14 +492,24 @@ class TestCreateServer(base.TestCase):
                          u'imageRef': u'image-id',
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
         ])
         self.cloud.create_server(
             'server-name',
             dict(id='image-id'), dict(id='flavor-id'), wait=True),
 
+        # This is a pretty dirty hack to ensure we in principle use object with
+        # expected properties
+        srv = server.Server.existing(
+            connection=self.cloud,
+            min_count=1, max_count=1,
+            networks='auto',
+            imageRef='image-id',
+            flavorRef='flavor-id',
+            **fake_server)
         mock_wait.assert_called_once_with(
-            fake_server,
+            srv,
             auto_ip=True, ips=None,
             ip_pool=None, reuse=True, timeout=180,
             nat_destination=None,
@@ -499,6 +531,7 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -509,8 +542,8 @@ class TestCreateServer(base.TestCase):
                          u'imageRef': u'image-id',
                          u'max_count': 1,
                          u'min_count': 1,
-                         u'name': u'server-name'}})),
-            self.get_nova_discovery_mock_dict(),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail']),
@@ -529,8 +562,8 @@ class TestCreateServer(base.TestCase):
                      'compute', 'public', append=['servers', '1234'])),
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'compute', 'public', append=['servers', 'detail']),
-                 json={'servers': []}),
+                     'compute', 'public', append=['servers', '1234']),
+                 status_code=404),
         ])
         mock_add_ips_to_server.return_value = fake_server
         self.cloud._SERVER_AGE = 0
@@ -555,8 +588,16 @@ class TestCreateServer(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks']),
+                     'network', 'public',
+                     append=['v2.0', 'networks', 'network-name']),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=network-name']),
                  json={'networks': [network]}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -600,8 +641,16 @@ class TestCreateServer(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks']),
+                     'network', 'public',
+                     append=['v2.0', 'networks', 'network-name']),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=network-name']),
                  json={'networks': [network]}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -644,6 +693,7 @@ class TestCreateServer(base.TestCase):
         fixed_ip = '10.0.0.1'
         build_server = fakes.make_fake_server('1234', '', 'BUILD')
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -686,6 +736,7 @@ class TestCreateServer(base.TestCase):
         fixed_ip = '10.0.0.1'
         build_server = fakes.make_fake_server('1234', '', 'BUILD')
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -730,6 +781,7 @@ class TestCreateServer(base.TestCase):
         fixed_ip = 'fe80::28da:5fff:fe57:13ed'
         build_server = fakes.make_fake_server('1234', '', 'BUILD')
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -833,6 +885,7 @@ class TestCreateServer(base.TestCase):
         port_id = uuid.uuid4().hex
 
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers']),
@@ -874,9 +927,10 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'compute', 'public', append=['os-volumes_boot']),
+                     'compute', 'public', append=['servers']),
                  json={'server': build_server},
                  validate=dict(
                      json={'server': {
@@ -900,7 +954,8 @@ class TestCreateServer(base.TestCase):
                                  u'uuid': u'volume001'
                              }
                          ],
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', '1234']),
@@ -926,9 +981,10 @@ class TestCreateServer(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public', append=['v2.0', 'networks']),
                  json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'compute', 'public', append=['os-volumes_boot']),
+                     'compute', 'public', append=['servers']),
                  json={'server': build_server},
                  validate=dict(
                      json={'server': {
@@ -943,7 +999,8 @@ class TestCreateServer(base.TestCase):
                              u'source_type': u'image',
                              u'uuid': u'image-id',
                              u'volume_size': u'1'}],
-                         u'name': u'server-name'}})),
+                         u'name': u'server-name',
+                         'networks': 'auto'}})),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', '1234']),
@@ -960,3 +1017,175 @@ class TestCreateServer(base.TestCase):
             wait=False)
 
         self.assert_calls()
+
+    def test_create_server_scheduler_hints(self):
+        """
+        Test that setting scheduler_hints will include them in POST request
+        """
+        scheduler_hints = {
+            'group': self.getUniqueString('group'),
+        }
+        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
+        fake_server['scheduler_hints'] = scheduler_hints
+
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'networks']),
+                 json={'networks': []}),
+            self.get_nova_discovery_mock_dict(),
+            dict(method='POST',
+                 uri=self.get_mock_url(
+                     'compute', 'public', append=['servers']),
+                 json={'server': fake_server},
+                 validate=dict(
+                     json={
+                         'server': {
+                             u'flavorRef': u'flavor-id',
+                             u'imageRef': u'image-id',
+                             u'max_count': 1,
+                             u'min_count': 1,
+                             u'name': u'server-name',
+                             'networks': 'auto'},
+                         u'OS-SCH-HNT:scheduler_hints': scheduler_hints, })),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'compute', 'public', append=['servers', '1234']),
+                 json={'server': fake_server}),
+        ])
+
+        self.cloud.create_server(
+            name='server-name', image=dict(id='image-id'),
+            flavor=dict(id='flavor-id'),
+            scheduler_hints=scheduler_hints, wait=False)
+
+        self.assert_calls()
+
+    def test_create_server_scheduler_hints_group_merge(self):
+        """
+        Test that setting both scheduler_hints and group results in merged
+        hints in POST request
+        """
+        group_id = uuid.uuid4().hex
+        group_name = self.getUniqueString('server-group')
+        policies = ['affinity']
+        fake_group = fakes.make_fake_server_group(
+            group_id, group_name, policies)
+
+        # The scheduler hints we pass in
+        scheduler_hints = {
+            'different_host': [],
+        }
+
+        # The scheduler hints we expect to be in POST request
+        scheduler_hints_merged = {
+            'different_host': [],
+            'group': group_id,
+        }
+
+        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
+        fake_server['scheduler_hints'] = scheduler_hints_merged
+
+        self.register_uris([
+            self.get_nova_discovery_mock_dict(),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'compute', 'public', append=['os-server-groups']),
+                 json={'server_groups': [fake_group]}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'networks']),
+                 json={'networks': []}),
+            dict(method='POST',
+                 uri=self.get_mock_url(
+                     'compute', 'public', append=['servers']),
+                 json={'server': fake_server},
+                 validate=dict(
+                     json={
+                         'server': {
+                             u'flavorRef': u'flavor-id',
+                             u'imageRef': u'image-id',
+                             u'max_count': 1,
+                             u'min_count': 1,
+                             u'name': u'server-name',
+                             'networks': 'auto'},
+                         u'OS-SCH-HNT:scheduler_hints': scheduler_hints_merged,
+                     })),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'compute', 'public', append=['servers', '1234']),
+                 json={'server': fake_server}),
+        ])
+
+        self.cloud.create_server(
+            name='server-name', image=dict(id='image-id'),
+            flavor=dict(id='flavor-id'),
+            scheduler_hints=dict(scheduler_hints), group=group_name,
+            wait=False)
+
+        self.assert_calls()
+
+    def test_create_server_scheduler_hints_group_override(self):
+        """
+        Test that setting group in both scheduler_hints and group param prefers
+        param
+        """
+        group_id_scheduler_hints = uuid.uuid4().hex
+        group_id_param = uuid.uuid4().hex
+        group_name = self.getUniqueString('server-group')
+        policies = ['affinity']
+        fake_group = fakes.make_fake_server_group(
+            group_id_param, group_name, policies)
+
+        # The scheduler hints we pass in that are expected to be ignored in
+        # POST call
+        scheduler_hints = {
+            'group': group_id_scheduler_hints,
+        }
+
+        # The scheduler hints we expect to be in POST request
+        group_scheduler_hints = {
+            'group': group_id_param,
+        }
+
+        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
+        fake_server['scheduler_hints'] = group_scheduler_hints
+
+        self.register_uris([
+            self.get_nova_discovery_mock_dict(),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'compute', 'public', append=['os-server-groups']),
+                 json={'server_groups': [fake_group]}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'networks']),
+                 json={'networks': []}),
+            dict(method='POST',
+                 uri=self.get_mock_url(
+                     'compute', 'public', append=['servers']),
+                 json={'server': fake_server},
+                 validate=dict(
+                     json={
+                         'server': {
+                             u'flavorRef': u'flavor-id',
+                             u'imageRef': u'image-id',
+                             u'max_count': 1,
+                             u'min_count': 1,
+                             u'name': u'server-name',
+                             'networks': 'auto'},
+                         u'OS-SCH-HNT:scheduler_hints': group_scheduler_hints,
+                     })),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'compute', 'public', append=['servers', '1234']),
+                 json={'server': fake_server}),
+        ])
+
+        self.cloud.create_server(
+            name='server-name', image=dict(id='image-id'),
+            flavor=dict(id='flavor-id'),
+            scheduler_hints=dict(scheduler_hints), group=group_name,
+            wait=False)
+
+        self.assert_calls()
