@@ -93,12 +93,14 @@ class TestPort(base.TestCase):
                               "mac_address": "mac_address",
                               "name": "name",
                               "network_id": "network_id",
+                              "security_groups": "security_groups",
                               "status": "status",
                               "subnet_id": "subnet_id",
                               "is_admin_state_up": "admin_state_up",
                               "is_port_security_enabled":
                                   "port_security_enabled",
                               "project_id": "project_id",
+                              "security_group_ids": "security_groups",
                               "limit": "limit",
                               "marker": "marker",
                               "any_tags": "tags-any",
